==> elfos-0.8.0.tar.gz: elfos/kernel/schedule.c <==
#include<kernel/thread.h>
#include<kernel/mm.h>
#include<arch/types.h>
#include<kernel/schedule.h>
#include<kernel/interrupt.h>
#include<drivers/serial/serial.h>

#ifdef DEBUG
#include<kernel/debug.h>
#endif //DEBUG

static tcb_t rdyListHead_g;
tcb_t *pCurrThread_g, *pNextThread_g; // Current tcb and Next tcb
waitList_t *pBlockedList_g;
policy sched_policy_g;

/*************************************************************************************
* Function Name : Kern_TickIsr()
* Date : `07. 4. 18
* Writen by : Kim pil jou
* Argument : No argument
* Return type : TRUE / FAIL
* Operation : This function is handle port's tick interrupt. tickdly decrease, if tickdly
			is zero, call Kern_Sched() and Schedule start.
*************************************************************************************/

#if 0
int8s 
Kern_TickIsr(){    //time ͷƮ 񽺷ƾ 
  pCurrThread_g->tickDly--;   //tick   
  if(pCurrThread_g->tickDly == 0){   //0  Ǿ 
    Kern_Sched();  //ٷ ȣ
	pCurrThread_g = pNextThread_g;     //next tcb   tcb  
  } 
  return TRUE;
} 
#endif 

/*************************************************************************************
* Function Name : Kern_FindNextTcb_t()
* Date : `07. 4. 18
* Writen by : Kim pil jou
* Argument : No Argument
* Return type : Next tcb_t
* Operation : This function is finding Nexttcb_t and return. finded Nexttcb_t inserted
*		ReadyList's tail. 
*
* Modified by : Sun Joon Gyu	(2007/05/27)
* 	I changed the list structure of thread control block.
* 	So, The codes for round-robin scheduling should also be changed.
*************************************************************************************/

tcb_t *
Kern_FindNextTcb_t()
{   //nexttcb ã Լ 

	tcb_t *pNextThread;
#ifdef SCHED_DRR
	struct list_head *tempList = rdyListHead_g.threadList.next;
	ListDel(rdyListHead_g.threadList.next);
	ListAddTail(tempList,&(rdyListHead_g.threadList));
	pNextThread = ListEntry(tempList,tcb_t,threadList);
#endif //SCHED_DRR
  return pNextThread;
   	
} 


/*************************************************************************************
* Function Name : Kern_Sched()
* Date : `07. 4. 18
* Writen by : Kim pil jou
* Argument : No Argument
* Return type : TRUE / FAIL
* Operation : This Function is Find Next tcb_t and port_ThreadSwitch() and Schedular start.
*************************************************************************************/


int8s 
Kern_Sched(){
  int32s i;
  int32u *sp;
Port_InterruptDisable();
  pNextThread_g = Kern_FindNextTcb_t();
  CurrentThread()->timeSlice= QUANTUM_TIME;
Port_InterruptEnable();
  Port_ThreadSwitch();
  return TRUE;
}


/*************************************************************************************
* Function Name : Kern_SchedualerStart()
* Date : `07. 4. 18
* Writen by : Kim pil jou
* Argument : No Argument
* Return type : TRUE / FAIL
* Operation : This Function is called Kern_Sched() and Schedular start.
*************************************************************************************/

int8s 
Kern_SchedulerStart(){
  
  Kern_Sched();

  return TRUE;

}


/*************************************************************************************
* Function Name : Kern_RdyListInit()
* Date : `07. 4. 18
* Writen by : Kim pil jou
* Argument : No Argument
* Return type : TRUE / FAIL
* Operation : This function is ReadyList initialise. this doing pRdyListhead_g and 
*		pRdyListTail_g connect. 
* 
* Modified by : Sun Joon Gyu	(2007/05/27)	
* 	I changed the list structure of thread control block.
* 	So, The codes for round-robin scheduling should also be changed.
*
*************************************************************************************/

int8s 
Kern_RdyListInit(){

	INIT_LIST_HEAD(&(rdyListHead_g.threadList));
  return TRUE;
}





/*************************************************************************** 
 * Function    : BlockListInsert()
 * Date        : 2007/04/30
 * Author      : Sun Joon Gyu
 * Parameter   : tcb_t *thread	- a thread to be blocked
 * Return value: 
 * Description : It adds the thread from parameter to the list 'pBlockedList_g'
 * ***************************************************************************/

int8s
BlockedListInsert(tcb_t *thread)
{

	ListAddTail(&(thread->threadList),&(pBlockedList_g->threadList));
	return TRUE;
}

tcb_t *
BlockedListRemove(tcb_t *thread)
{
	return ;
}



/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int8s
Kern_BlockedListInit()
{
	SerialOutputString("Start to allocate memory ...\n");
	if((pBlockedList_g = (waitList_t *)MemAlloc(sizeof(struct waitList))) == NULL){
			SerialOutputString("Failed to allocate memory...\n");
	}
	SerialOutputString("memory allocated...\n");

	INIT_LIST_HEAD(&pBlockedList_g->threadList);
	return TRUE;

}


/*************************************************************************** 
 * Function    : Kern_GetCurrThread()
 * Date        : 
 * Author      : Joongyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

tcb_t *
Kern_GetCurrThread()
{
	return pCurrThread_g;
}


int8s 
Kern_RdyListInsert(tcb_t * newtcb)
{

	ListAddTail(&newtcb->threadList, &rdyListHead_g.threadList);
	return ;
}


/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

void 
Sleep(int32u time)
{
/* ENTER_CRITICAL_SECTION(); */
	tcb_t *curr = Kern_GetCurrThread();
	curr->tickDly = time;

	Kern_BlockThread(curr,pBlockedList_g);
/* EXIT_CRITICAL_SECTION(); */

	Kern_Sched();
}




/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int8s
Kern_BlockThread(tcb_t *curr,waitList_t *pList)
{
	ListDel(&curr->threadList);
	ListAddTail(&curr->threadList,&pList->threadList);




}




/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int8s 
Kern_WakeThread(tcb_t *thread)
{
#ifdef SCHED_DRR
	ListDel(&thread->threadList);
	ListAddTail(&thread->threadList,&rdyListHead_g.threadList);
#endif //SCHED_DRR

}



void 
TaskStart(void *data)
{
  SerialOutputString("-----------\n");
  ((void (*)())data)();
}


int32u
SetPolicy_Default(){
#if SCHED_DRR & !SCHED_PRIO & !SCHED_EDF
  sched_policy_g = D_RR;
#elif SCHED_PRIO & !SCHED_DRR & !SCHED_EDF /*Not implemented yet */
  sched_policy_g = PRIO;
#elif SCHED_EDF & !SCHED_PRIO & !SHCED_DRR /*Not implemented yet */
  shced_policy_g = EDF;
#else
  return FALSE;
#endif //SCHED_*
  return TRUE;
}

int32u
SetPolicy(policy new)
{
		if(new >= MAX_POLICY && new < MIN_POLICY)
				return FALSE;

		sched_policy_g = new;
		return TRUE;
}

void
Kern_SchedInit() 
{
	if(SetPolicy_Default() != TRUE){
			/*This error is critical. So print a panic message to user
			 * and halt it */
			SerialOutputString("Unkown scheduling policy choosed.\n");
			SerialOutputString("Check scheduling policy with Makefile.\n");
			while(1);	/*TODO: It should be replaced by a instruction 'halt' */
	}
	SerialOutputString("Policy Selected...\n");
	Kern_RdyListInit();
	SerialOutputString("Ready List Initialized...\n");
	Kern_BlockedListInit();
	SerialOutputString("Blocked List Initialized...\n");
	return ;
}

==> elfos-0.8.8.tar.gz: elfos/kernel/schedule.c <==
#include<kernel/thread.h>
#include<kernel/mm.h>
#include<arch/types.h>
#include<kernel/schedule.h>
#include<kernel/interrupt.h>
#include<drivers/serial/serial.h>
#include<kernel/list.h>

#ifdef DEBUG
#include<kernel/debug.h>
#endif //DEBUG

static tcb_t rdyListHead_g;
tcb_t *pCurrThread_g, *pNextThread_g; // Current tcb and Next tcb
waitList_t *pBlockedList_g;
policy sched_policy_g;

/*************************************************************************************
* Function Name : Kern_TickIsr()
* Date : `07. 4. 18
* Writen by : Kim pil jou
* Argument : No argument
* Return type : TRUE / FAIL
* Operation : This function is handle port's tick interrupt. tickdly decrease, if tickdly
			is zero, call Kern_Sched() and Schedule start.
*************************************************************************************/

#if 0
int8s 
Kern_TickIsr(){    //time ͷƮ 񽺷ƾ 
  pCurrThread_g->tickDly--;   //tick   
  if(pCurrThread_g->tickDly == 0){   //0  Ǿ 
    Kern_Sched();  //ٷ ȣ
	pCurrThread_g = pNextThread_g;     //next tcb   tcb  
  } 
  return TRUE;
} 
#endif 

/*************************************************************************************
* Function Name : Kern_FindNextTcb_t()
* Date : `07. 4. 18
* Writen by : Kim pil jou
* Argument : No Argument
* Return type : Next tcb_t
* Operation : This function is finding Nexttcb_t and return. finded Nexttcb_t inserted
*		ReadyList's tail. 
*
* Modified by : Sun Joon Gyu	(2007/05/27)
* 	I changed the list structure of thread control block.
* 	So, The codes for round-robin scheduling should also be changed.
*************************************************************************************/

tcb_t *
Kern_FindNextTcb_t()
{   //nexttcb ã Լ 

	tcb_t *pNextThread;
	policy *pcurr_policy = GetSchedPolicy();

	if((*pcurr_policy <= MIN_POLICY) && (*pcurr_policy >= MAX_POLICY)){
			/*Scheduling Panic */
			while(1);
	}

	switch(*pcurr_policy){
			case D_RR : 
					{
							struct list_head *tempList = rdyListHead_g.threadList.next;
							ListDel(rdyListHead_g.threadList.next);
							ListAddTail(tempList,&(rdyListHead_g.threadList));
							pNextThread = ListEntry(tempList,tcb_t,threadList);
					}
					break;
			case PRIO :
					break;
			case EDF :
					break;
	}
  return pNextThread;
   	
} 



/*************************************************************************************
* Function Name : Kern_Sched()
* Date : `07. 4. 18
* Writen by : Kim pil jou
* Argument : No Argument
* Return type : TRUE / FAIL
* Operation : This Function is Find Next tcb_t and port_ThreadSwitch() and Schedular start.
*************************************************************************************/


int8s 
Kern_Sched(){
  int32s i;
  int32u *sp;
Port_InterruptDisable();
  pNextThread_g = Kern_FindNextTcb_t();
  CurrentThread()->timeSlice= QUANTUM_TIME;
Port_InterruptEnable();
  Port_ThreadSwitch();
  return TRUE;
}


/*************************************************************************************
* Function Name : Kern_SchedualerStart()
* Date : `07. 4. 18
* Writen by : Kim pil jou
* Argument : No Argument
* Return type : TRUE / FAIL
* Operation : This Function is called Kern_Sched() and Schedular start.
*************************************************************************************/

int8s 
Kern_SchedulerStart(){
  
  Kern_Sched();

  return TRUE;

}


/*************************************************************************************
* Function Name : Kern_RdyListInit()
* Date : `07. 4. 18
* Writen by : Kim pil jou
* Argument : No Argument
* Return type : TRUE / FAIL
* Operation : This function is ReadyList initialise. this doing pRdyListhead_g and 
*		pRdyListTail_g connect. 
* 
* Modified by : Sun Joon Gyu	(2007/05/27)	
* 	I changed the list structure of thread control block.
* 	So, The codes for round-robin scheduling should also be changed.
*
*************************************************************************************/

int8s 
Kern_RdyListInit(){

	INIT_LIST_HEAD(&(rdyListHead_g.threadList));
  return TRUE;
}





/*************************************************************************** 
 * Function    : BlockListInsert()
 * Date        : 2007/04/30
 * Author      : Sun Joon Gyu
 * Parameter   : tcb_t *thread	- a thread to be blocked
 * Return value: 
 * Description : It adds the thread from parameter to the list 'pBlockedList_g'
 * ***************************************************************************/

int8s
BlockedListInsert(tcb_t *thread)
{

	ListAddTail(&(thread->threadList),&(pBlockedList_g->threadList));
	return TRUE;
}

tcb_t *
BlockedListRemove(tcb_t *thread)
{
	return ;
}



/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int8s
Kern_BlockedListInit()
{
	SerialOutputString("Start to allocate memory ...\n");
	if((pBlockedList_g = (waitList_t *)MemAlloc(sizeof(struct waitList))) == NULL){
			SerialOutputString("Failed to allocate memory...\n");
	}
	SerialOutputString("memory allocated...\n");

	INIT_LIST_HEAD(&pBlockedList_g->threadList);
	return TRUE;

}


/*************************************************************************** 
 * Function    : Kern_GetCurrThread()
 * Date        : 
 * Author      : Joongyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

tcb_t *
Kern_GetCurrThread()
{
	return pCurrThread_g;
}


int8s 
Kern_RdyListInsert(tcb_t * newtcb)
{

	ListAddTail(&newtcb->threadList, &rdyListHead_g.threadList);
	return ;
}


/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

void 
Sleep(int32u time)
{
/* ENTER_CRITICAL_SECTION(); */
Port_InterruptDisable();
	tcb_t *curr = Kern_GetCurrThread();
	curr->tickDly = time;

	Kern_BlockThread(curr,pBlockedList_g);
Port_InterruptEnable();
/* EXIT_CRITICAL_SECTION(); */

	Kern_Sched();
}




/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int8s
Kern_BlockThread(tcb_t *curr,waitList_t *pList)
{
	ListDel(&curr->threadList);
	ListAddTail(&curr->threadList,&pList->threadList);




}




/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int8s 
Kern_WakeThread(tcb_t *thread)
{
#ifdef SCHED_DRR
	ListDel(&thread->threadList);
	ListAddTail(&thread->threadList,&rdyListHead_g.threadList);
#endif //SCHED_DRR

}



void 
TaskStart(void *data)
{
  SerialOutputString("-----------\n");
  ((void (*)())data)();
}


int32u
SetPolicy_Default(){
#if SCHED_DRR & !SCHED_PRIO & !SCHED_EDF
  sched_policy_g = D_RR;
#elif SCHED_PRIO & !SCHED_DRR & !SCHED_EDF /*Not implemented yet */
  sched_policy_g = PRIO;
#elif SCHED_EDF & !SCHED_PRIO & !SHCED_DRR /*Not implemented yet */
  shced_policy_g = EDF;
#else
  return FALSE;
#endif //SCHED_*
  return TRUE;
}

int32u
SetPolicy(policy new)
{
		if(new >= MAX_POLICY && new <= MIN_POLICY)
				return FALSE;

		sched_policy_g = new;
		return TRUE;
}

void
Kern_SchedInit() 
{
	if(SetPolicy_Default() != TRUE){
			/*This error is critical. So print a panic message to user
			 * and halt it */
			SerialOutputString("Unkown scheduling policy choosed.\n");
			SerialOutputString("Check scheduling policy with Makefile.\n");
			while(1);	/*TODO: It should be replaced by a instruction 'halt' */
	}
	SerialOutputString("Policy Selected...\n");
	Kern_RdyListInit();
	SerialOutputString("Ready List Initialized...\n");
	Kern_BlockedListInit();
	SerialOutputString("Blocked List Initialized...\n");
	return ;
}
